@@ -8,8 +8,8 @@
 #include "datadevice_interface.h"
 #include "datadevice_interface_p.h"
 #include "datadevicemanager_interface.h"
-#include "datasource_interface.h"
 #include "dataoffer_interface.h"
+#include "datasource_interface.h"
 #include "display.h"
 #include "pointer_interface.h"
 #include "seat_interface.h"
@@ -19,7 +19,6 @@
 
 namespace KWaylandServer
 {
-
 class DragAndDropIconPrivate : public SurfaceRole
 {
 public:
@@ -40,8 +39,8 @@ void DragAndDropIconPrivate::commit()
     position += surface()->offset();
 }
 
-DragAndDropIcon::DragAndDropIcon(SurfaceInterface *surface, QObject *parent)
-    : QObject(parent)
+DragAndDropIcon::DragAndDropIcon(SurfaceInterface *surface)
+    : QObject(surface)
     , d(new DragAndDropIconPrivate(surface))
 {
 }
@@ -72,20 +71,17 @@ DataDeviceInterfacePrivate::DataDeviceInterfacePrivate(SeatInterface *seat, Data
 {
 }
 
-void DataDeviceInterfacePrivate::endDrag()
-{
-    icon.reset();
-}
-
-void DataDeviceInterfacePrivate::data_device_start_drag(Resource *resource, wl_resource *sourceResource, wl_resource *originResource, wl_resource *iconResource, uint32_t serial)
+void DataDeviceInterfacePrivate::data_device_start_drag(Resource *resource,
+                                                        wl_resource *sourceResource,
+                                                        wl_resource *originResource,
+                                                        wl_resource *iconResource,
+                                                        uint32_t serial)
 {
     SurfaceInterface *iconSurface = SurfaceInterface::get(iconResource);
 
     const SurfaceRole *surfaceRole = SurfaceRole::get(iconSurface);
     if (surfaceRole) {
-        wl_resource_post_error(resource->handle, error_role,
-                               "the icon surface already has a role assigned %s",
-                               surfaceRole->name().constData());
+        wl_resource_post_error(resource->handle, error_role, "the icon surface already has a role assigned %s", surfaceRole->name().constData());
         return;
     }
 
@@ -95,10 +91,6 @@ void DataDeviceInterfacePrivate::data_device_start_drag(Resource *resource, wl_r
         dataSource = DataSourceInterface::get(sourceResource);
     }
 
-    if (proxyRemoteSurface) {
-        // origin is a proxy surface
-        focusSurface = proxyRemoteSurface.data();
-    }
     const bool pointerGrab = seat->hasImplicitPointerGrab(serial) && seat->focusedPointerSurface() == focusSurface;
     if (!pointerGrab) {
         // Client doesn't have pointer grab.
@@ -108,18 +100,14 @@ void DataDeviceInterfacePrivate::data_device_start_drag(Resource *resource, wl_r
             return;
         }
     }
-    // TODO: source is allowed to be null, handled client internally!
-    source = dataSource;
-    if (dataSource) {
-        QObject::connect(dataSource, &AbstractDataSource::aboutToBeDestroyed, q, [this] { source = nullptr; });
-    }
+
+    DragAndDropIcon *dragIcon = nullptr;
     if (iconSurface) {
-        icon.reset(new DragAndDropIcon(iconSurface));
-        QObject::connect(iconSurface, &SurfaceInterface::aboutToBeDestroyed, icon.data(), [this] { icon.reset(); });
+        // drag icon lifespan is mapped to surface lifespan
+        dragIcon = new DragAndDropIcon(iconSurface);
     }
-    surface = focusSurface;
     drag.serial = serial;
-    Q_EMIT q->dragStarted();
+    Q_EMIT q->dragStarted(dataSource, focusSurface, serial, dragIcon);
 }
 
 void DataDeviceInterfacePrivate::data_device_set_selection(Resource *resource, wl_resource *source, uint32_t serial)
@@ -152,7 +140,6 @@ void DataDeviceInterfacePrivate::data_device_release(QtWaylandServer::wl_data_de
     wl_resource_destroy(resource->handle);
 }
 
-
 DataOfferInterface *DataDeviceInterfacePrivate::createDataOffer(AbstractDataSource *source)
 {
     if (!source) {
@@ -180,7 +167,7 @@ void DataDeviceInterfacePrivate::data_device_destroy_resource(QtWaylandServer::w
 }
 
 DataDeviceInterface::DataDeviceInterface(SeatInterface *seat, wl_resource *resource)
-    : QObject(nullptr)
+    : AbstractDropHandler(nullptr)
     , d(new DataDeviceInterfacePrivate(seat, this, resource))
 {
     SeatInterfacePrivate *seatPrivate = SeatInterfacePrivate::get(seat);
@@ -194,21 +181,6 @@ SeatInterface *DataDeviceInterface::seat() const
     return d->seat;
 }
 
-DataSourceInterface *DataDeviceInterface::dragSource() const
-{
-    return d->source;
-}
-
-DragAndDropIcon *DataDeviceInterface::icon() const
-{
-    return d->icon.data();
-}
-
-SurfaceInterface *DataDeviceInterface::origin() const
-{
-    return d->proxyRemoteSurface ? d->proxyRemoteSurface.data() : d->surface;
-}
-
 DataSourceInterface *DataDeviceInterface::selection() const
 {
     return d->selection;
@@ -238,7 +210,6 @@ void DataDeviceInterface::drop()
     disconnect(d->drag.destroyConnection);
     d->drag.destroyConnection = QMetaObject::Connection();
     d->drag.surface = nullptr;
-    wl_client_flush(d->resource()->client());
 }
 
 void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, quint32 serial)
@@ -264,92 +235,70 @@ void DataDeviceInterface::updateDragTarget(SurfaceInterface *surface, quint32 se
         }
         // don't update serial, we need it
     }
-    auto dragSourceDevice = d->seat->dragSource();
-    if (!surface || !dragSourceDevice) {
-        if (auto s = dragSourceDevice->dragSource()) {
+    auto dragSource = d->seat->dragSource();
+    if (!surface || !dragSource) {
+        if (auto s = dragSource) {
             s->dndAction(DataDeviceManagerInterface::DnDAction::None);
         }
         return;
     }
-    if (d->proxyRemoteSurface && d->proxyRemoteSurface == surface) {
-        // A proxy can not have the remote surface as target.
-        // TODO: do this for all client's surfaces?
-        return;
+
+    if (dragSource) {
+        dragSource->accept(QString());
     }
-    auto *source = dragSourceDevice->dragSource();
-    if (source) {
-        source->setAccepted(false);
-    }
-    DataOfferInterface *offer = d->createDataOffer(source);
+    DataOfferInterface *offer = d->createDataOffer(dragSource);
     d->drag.surface = surface;
     if (d->seat->isDragPointer()) {
-        d->drag.posConnection = connect(d->seat, &SeatInterface::pointerPosChanged, this,
-            [this] {
-                const QPointF pos = d->seat->dragSurfaceTransformation().map(d->seat->pointerPos());
-                d->send_motion(d->seat->timestamp(),
-                                        wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
-                wl_client_flush(d->resource()->client());
-            }
-        );
+        d->drag.posConnection = connect(d->seat, &SeatInterface::pointerPosChanged, this, [this] {
+            const QPointF pos = d->seat->dragSurfaceTransformation().map(d->seat->pointerPos());
+            d->send_motion(d->seat->timestamp(), wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+        });
     } else if (d->seat->isDragTouch()) {
-        d->drag.posConnection = connect(d->seat, &SeatInterface::touchMoved, this,
-            [this](qint32 id, quint32 serial, const QPointF &globalPosition) {
-                Q_UNUSED(id);
-                if (serial != d->drag.serial) {
-                    // different touch down has been moved
-                    return;
-                }
-                const QPointF pos = d->seat->dragSurfaceTransformation().map(globalPosition);
-                d->send_motion(d->seat->timestamp(),
-                               wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
-                wl_client_flush(d->resource()->client());
+        d->drag.posConnection = connect(d->seat, &SeatInterface::touchMoved, this, [this](qint32 id, quint32 serial, const QPointF &globalPosition) {
+            Q_UNUSED(id);
+            if (serial != d->drag.serial) {
+                // different touch down has been moved
+                return;
             }
-        );
+            const QPointF pos = d->seat->dragSurfaceTransformation().map(globalPosition);
+            d->send_motion(d->seat->timestamp(), wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+        });
     }
-    d->drag.destroyConnection = connect(d->drag.surface, &QObject::destroyed, this,
-        [this] {
-            d->send_leave();
-            if (d->drag.posConnection) {
-                disconnect(d->drag.posConnection);
-            }
-            d->drag = DataDeviceInterfacePrivate::Drag();
+    d->drag.destroyConnection = connect(d->drag.surface, &QObject::destroyed, this, [this] {
+        d->send_leave();
+        if (d->drag.posConnection) {
+            disconnect(d->drag.posConnection);
         }
-    );
+        d->drag = DataDeviceInterfacePrivate::Drag();
+    });
 
     // TODO: handle touch position
     const QPointF pos = d->seat->dragSurfaceTransformation().map(d->seat->pointerPos());
-    d->send_enter(serial, surface->resource(),
-                  wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()), offer ? offer->resource() : nullptr);
+    d->send_enter(serial, surface->resource(), wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()), offer ? offer->resource() : nullptr);
     if (offer) {
         offer->sendSourceActions();
-        auto matchOffers = [source, offer] {
+        auto matchOffers = [dragSource, offer] {
             DataDeviceManagerInterface::DnDAction action{DataDeviceManagerInterface::DnDAction::None};
-            if (source->supportedDragAndDropActions().testFlag(offer->preferredDragAndDropAction())) {
+            if (dragSource->supportedDragAndDropActions().testFlag(offer->preferredDragAndDropAction())) {
                 action = offer->preferredDragAndDropAction();
             } else {
-                if (source->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Copy) &&
-                    offer->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Copy)) {
+                if (dragSource->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Copy)
+                    && offer->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Copy)) {
                     action = DataDeviceManagerInterface::DnDAction::Copy;
-                } else if (source->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Move) &&
-                    offer->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Move)) {
+                } else if (dragSource->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Move)
+                           && offer->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Move)) {
                     action = DataDeviceManagerInterface::DnDAction::Move;
-                } else if (source->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Ask) &&
-                    offer->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Ask)) {
+                } else if (dragSource->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Ask)
+                           && offer->supportedDragAndDropActions().testFlag(DataDeviceManagerInterface::DnDAction::Ask)) {
                     action = DataDeviceManagerInterface::DnDAction::Ask;
                 }
             }
             offer->dndAction(action);
-            source->dndAction(action);
+            dragSource->dndAction(action);
         };
-        d->drag.targetActionConnection = connect(offer, &DataOfferInterface::dragAndDropActionsChanged, source, matchOffers);
-        d->drag.sourceActionConnection = connect(source, &DataSourceInterface::supportedDragAndDropActionsChanged, source, matchOffers);
+        d->drag.targetActionConnection = connect(offer, &DataOfferInterface::dragAndDropActionsChanged, dragSource, matchOffers);
+        d->drag.sourceActionConnection = connect(dragSource, &AbstractDataSource::supportedDragAndDropActionsChanged, dragSource, matchOffers);
     }
-    wl_client_flush(d->resource()->client());
-}
-
-quint32 DataDeviceInterface::dragImplicitGrabSerial() const
-{
-    return d->drag.serial;
 }
 
 void DataDeviceInterface::updateProxy(SurfaceInterface *remote)
